package rabinizer.exec;


import java.util.Map;

import rabinizer.formulas.Literal;
import net.sf.javabdd.*;

/**
 * Service class that serves as an interface to javabdd.
 * @author andreas
 *
 */
public class BDDService {

	private static BDDFactory bf;
	
	public static void init(int numOfVar){
		bf = BDDFactory.init("java", numOfVar+1000, 1000);
		bf.setVarNum(numOfVar);
	}

	public static BDD getTrueBDD() {
		return bf.one();
	}
	
	public static BDD getFalseBDD() {
		return bf.zero();
	}

	public static BDD variableToBDD(int i) {
		return bf.ithVar(i);
	}

	public static BDD valuationToBDD(Valuation v) {
		BDD result = bf.one();  // BDD for True
		for(Integer i:v.keySet()){
			if(v.get(i))
				result = result.and(variableToBDD(i));
			else
				result = result.and(variableToBDD(i).not());
		}
		return result;
	}

	public static BDD valuationSetToBDD(ValuationSet vs) {
		BDD result = getFalseBDD();
		for(Valuation v : vs) {
			result = result.or(valuationToBDD(v));
		}
		return result;
	}
	
	
//			if (!b.and(BDDService.getVariableBDD(i)).isZero()) {
	
	public static String variableToString(int var) {
		String result = Misc.bijectionIdAtom.atom(var);
		return result;
	}

	public static String BDDtoString(BDD b) {
		if (b.isOne())
			return "tt";
		else if (b.isZero())
			return "ff";
		else {
			boolean firstOp = false;
			String result = "";
			if (b.high().isOne()) {
				result += variableToString(b.level()) + "";
				firstOp = true;
			} else if (!b.high().isZero()) {
				result += "(" + variableToString(b.level()) + "&"
						+ BDDtoString(b.high()) + ")";
				firstOp = true;
			}

			if (b.low().isOne()) {
				result += (firstOp ? "+" : "") + "!"
						+ variableToString(b.level()) + "";
			} else if (!b.low().isZero())
				result += (firstOp ? "+" : "") + "(!"
						+ variableToString(b.level()) + "&"
						+ BDDtoString(b.low()) + ")";
			result += "";
			return result;
		}
	}
}
