package rabinizer.exec;

import java.util.*;
import java.util.Map.Entry;
import rabinizer.formulas.*;

/**
 * 
 * @author Ruslan Ledesma-Garza
 * 
 */

public class DFA {
	
    /*
      second_component := List<Set<Formula>>, List<Set<Formula>>
      state := Tuple<Formula, second_component> := Tuple<Formula, List<Set<Formula>>>
    */
    public Set<Tuple<Formula, List<Set<Formula>>>> states = null;
    public Tuple<Formula, List<Set<Formula>>> initialState = null;
    public Map<Tuple<Tuple<Formula, List<Set<Formula>>>, Tuple<Formula, List<Set<Formula>>>>, ValuationSet> edges = null;
    public Map<Formula, Formula> displayLabels = null;
    
    // GeneralizedRabinAcceptanceCondition.class uses the following three
    public List<SegmentAutomaton> segmentAutomata = null;
    public Formula phi = null;
    public Map<Formula, Integer> rFtoSAid = null;
	
    /**
     * Algorithm DFA
     * 
     * @param phi
     */
    public DFA(Formula phi, List<SegmentAutomaton> sAs, List<Set<Formula>> iSSs, List<Valuation> valuations) {
    	
    	Misc.verboseln("DFA for " + phi);

    	// bundle the segmentAutomata, SA identifiers, & phi for use in Generalized Rabin Acceptance Condition
    	segmentAutomata = sAs;
    	rFtoSAid = new HashMap<Formula, Integer>();
    	for (int i = 0; i < sAs.size(); i++)
    		rFtoSAid.put(sAs.get(i).initialState, i);
    	this.phi = phi;

        // initial state
//        List<Set<Formula>> initialSegmentStates = new ArrayList<Set<Formula>>(sAs.size());
//        for (int i = 0; i < sAs.size(); i++) {
//        	Set<Formula> s = new HashSet<Formula>();
//        }
//        for (SegmentAutomaton sA : sAs) {
//            Set<Formula> s = new HashSet<Formula>();
//            s.add(sA.initialState);
//            initialSegmentStates.add(s);
//        }
        initialState = new Tuple<Formula, List<Set<Formula>>>(phi.unfold().representative(), iSSs);

        states = new HashSet<Tuple<Formula, List<Set<Formula>>>>();
        states.add(initialState);
        edges = new HashMap<Tuple<Tuple<Formula, List<Set<Formula>>>, Tuple<Formula, List<Set<Formula>>>>, ValuationSet>();
        Stack<Tuple<Formula, List<Set<Formula>>>> workstack = new Stack<Tuple<Formula, List<Set<Formula>>>>();
        displayLabels = new HashMap<Formula, Formula>();
        displayLabels.put(phi.unfold().representative(), phi);
        workstack.push(initialState);

        int loopCount = 0;
        while (!workstack.empty()) {
        
        	Misc.verboseln("loop count: " + loopCount++);
        	Misc.verboseln("workstack : " + workstack);
        	Misc.verboseln("workcount : " + workstack.size());
        	
            Tuple<Formula, List<Set<Formula>>> curr = workstack.pop();
            Misc.verboseln("Curr: " + curr);
            for (Valuation v : valuations) {
                Misc.verboseln("\tv: " + v);

                // successor state
                Formula displayLabel = curr.left.temporalStep(v);
                Formula stateLabel = displayLabel.unfold().representative();
                List<Set<Formula>> succSegmentStates = new ArrayList<Set<Formula>>(sAs.size());
                for (int i = 0; i < sAs.size(); i++) {
                	if (displayLabel.hasSubformula(sAs.get(i).initialState)) {
                		Set<Formula> cSS = curr.right.get(i);
                		Misc.verboseln("cSS: " + cSS);
                		Set<Formula> nSS = sAs.get(i).step(cSS, v);
                		Misc.verboseln("nSS: " + nSS);
                		succSegmentStates.add(nSS);
                	} else
                		succSegmentStates.add(new HashSet<Formula>());
                }
                Tuple<Formula, List<Set<Formula>>> succ = new Tuple<Formula, List<Set<Formula>>>(stateLabel, succSegmentStates);
                Misc.verboseln("\tSucc: " + succ);
                
                Tuple<Tuple<Formula, List<Set<Formula>>>, Tuple<Formula, List<Set<Formula>>>> currAndSucc = new Tuple<Tuple<Formula, List<Set<Formula>>>, Tuple<Formula, List<Set<Formula>>>>(curr, succ);
                if (edges.containsKey(currAndSucc)) {
                    Misc.verboseln("\tUpdate edge");
                    ValuationSet label = edges.get(currAndSucc);
                    label.add(v);
                    edges.put(currAndSucc, label);
                } else {
                    Misc.verboseln("\tNew edge");
                    ValuationSet label = new ValuationSet();
                    label.add(v);
                    edges.put(currAndSucc, label);
                }
                if (!states.contains(succ)) {
                	Misc.verboseln("\tNew state");
                    states.add(succ);
                    workstack.push(succ);
                }
                if (!displayLabels.containsKey(stateLabel)) {
                	Misc.verboseln("\tNew display label: " + displayLabel);
                	displayLabels.put(stateLabel, displayLabel);
                }
            }
        }
        Misc.verboseln("# states     : " + states.size());
        Misc.verboseln("# edges      : " + edges.size());
        Misc.verboseln("Initial state: " + initialState);
        Misc.verboseln("States       : " + states);
        Misc.verboseln("Edges        : " + edges);
    }

    public String toDotty() {
        String dot = "digraph \"GenRabinAutomaton " + initialState +"\" {\n";
//        Misc.verboseln("States       : " + states);
//        Misc.verboseln("# states     : " + states.size());
        // The initial state in the set states may be different from the states in the set of edges. This can happen
        // because HashSet does not guarantee that two segment states that are equal have the same string representation.
        // To circumvent the problem, use the same string representation for rendering nodes and edges in the dotty file.
        Map<Tuple<Formula, List<Set<Formula>>>, Tuple<Formula, List<Set<Formula>>>> normalStates = new HashMap<Tuple<Formula, List<Set<Formula>>>, Tuple<Formula, List<Set<Formula>>>>();        
        for (Tuple<Formula, List<Set<Formula>>> s : states) {
        	normalStates.put(s,s);
        	Misc.verbose(",");
        	Formula display = displayLabels.get(s.left);
        	String display2 = "";
        	for(Set<Formula> ss : s.right){
        		display2 += "[";
        		for(Formula sss:ss)
        			display2 += sss + ",";
        		display2 += "] ";
        	}
        	if (s.equals(initialState)) {
                dot += "node [shape=oval, label=\"" + display + " :: " + display2 + "\"]\"" + s + "\";\n";
            } else {
                dot += "node [shape=rectangle, label=\"" + display + " :: " + display2 + "\"]\"" + s + "\";\n";
            }
        }
        for (Entry<Tuple<Tuple<Formula, List<Set<Formula>>>, Tuple<Formula, List<Set<Formula>>>>, ValuationSet> edge : edges.entrySet()) {
        	Misc.verbose(".");
            dot += "\"" + normalStates.get(edge.getKey().left) + "\" -> \"" + normalStates.get(edge.getKey().right) + "\" [label=\"" + edge.getValue() + "\"];\n"; 
        }
        return dot + "}\n";
    }
}
