package rabinizer.exec;
import java.util.*;

import rabinizer.formulas.*;

public class RabinAcceptanceCondition {
	
	/*
	F  : set of states of the gen rabin automaton
	I  : set of states of the gen rabin automaton
	Is : list of set of states of the gen rabin automaton
	F = Set<Tuple<Formula, List<Set<Formula>>>>
	I = Set<Tuple<Formula, List<Set<Formula>>>>
	Is = List<Set<Tuple<Formula, List<Set<Formula>>>>>
	
	F2 = Set<Tuple<F, List<Integer>>>
	F2 = Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>

	I2 = Set<Tuple<I, List<Integer>>>
	I2 = Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>
	
	C = Set<Tuple<F2, I2>>
	C = Set<Tuple<Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>, Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>>>
	*/
	       
	public Set<Tuple<Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>, Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>>> condition = new HashSet<Tuple<Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>, Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>>>();
		
	RabinAcceptanceCondition(List<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Set<Tuple<Formula, List<Set<Formula>>>>>>> grac){
		List<Integer> lens = new ArrayList<Integer>(grac.size());
		for (int i = 0; i < grac.size(); i++)
			lens.add(i, grac.get(i).right.size());
		for (int i = 0; i < grac.size(); i++) {
			Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>> avoid = new HashSet<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>();
			List<Integer> initCoords = new ArrayList<Integer>(grac.size());
			for (int j = 0; j < grac.size(); j++)
				initCoords.add(j,0);
			avoidOfGenAvoid(grac.get(i).left, lens, lens.size()-1, initCoords, avoid);
			Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>> reach = new HashSet<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>();
			reachOfGenReach(grac.get(i).right.get(0), lens, lens.size()-1, i, initCoords, reach);
			condition.add(new Tuple<Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>, Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>>(avoid, reach));
		}
	}
	
	public void avoidOfGenAvoid(Set<Tuple<Formula, List<Set<Formula>>>> F, List<Integer> lens, int dimension, List<Integer> acc, Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>> avoid) {
		if (dimension < 0) {
			List<Integer> coords = new ArrayList<Integer>(acc.size());
			coords.addAll(acc);
			avoid.add(new Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>(F, coords));
		} else {
			for (int i = 0; i < lens.get(dimension); i++) {
				acc.set(dimension, i);
				avoidOfGenAvoid(F, lens, dimension-1, acc, avoid);
			}
		}
	}
	
	public void reachOfGenReach(Set<Tuple<Formula, List<Set<Formula>>>> I, List<Integer> lens, int dimension, int planeDim, List<Integer> acc, Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>> reach) {
		if (dimension < 0) {
			List<Integer> coords = new ArrayList<Integer>(acc.size());
			coords.addAll(acc);
//			Misc.verboseln("Reach state : "+I+", "+coords);
			reach.add(new Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>(I, coords));
		} else {
			if (dimension == planeDim) {
				acc.set(dimension, 0);
				reachOfGenReach(I, lens, dimension-1, planeDim, acc, reach);
			} else 
				for (int i = 0; i < lens.get(dimension); i++) {
					acc.set(dimension, i);
					reachOfGenReach(I, lens, dimension-1, planeDim, acc, reach);
				}
		}
	}
	
	public String toString() {
        int rpc = 0;
        String r = "Rabin pair count: "+condition.size()+"\n";
        for (Tuple<Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>, Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>> pair : condition) {
            r += "Pair "+(++rpc)+":\n";
        	r += pairToString(pair);
        }
        return r;
    }
    
	
    private String pairToString(Tuple<Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>, Set<Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>>>> pair) {
    	String r = "[\n";
        for (Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>> avoidRabinState : pair.left)
            r += " "+avoidRabinState+"\n";
        r += "]\n[\n";
        for (Tuple<Set<Tuple<Formula, List<Set<Formula>>>>, List<Integer>> reachRabinState : pair.right) {
        	r += " [\n";
        	for (Tuple<Formula, List<Set<Formula>>> reachGenRabinState : reachRabinState.left)
        		r += "  "+reachGenRabinState+"\n";
        	r += " ]" + reachRabinState.right+"\n";
        }
        r += "]\n";
        return r;
    }
    
    public int size() {
    	return condition.size();
    }
}
