package rabinizer.formulas;

import java.util.*;

import net.sf.javabdd.*;
import rabinizer.exec.*;

/**
 * @author Andreas Gaiser & Ruslan Ledesma-Garza
 *
 */
public class GOperator extends Formula {

	public Formula operand;
	
	public GOperator(Formula f) {
		this.operand = f;
	}
	
	public BDD bdd() {
		if (cachedBdd == null) {
			Formula booleanAtom = new GOperator(operand.representative());
			int bddVar = Misc.bijectionBooleanAtomBddVar.id(booleanAtom);
			if (Misc.bddFactory.varNum() <= bddVar) Misc.bddFactory.extVarNum(bddVar);
			cachedBdd = Misc.bddFactory.ithVar(bddVar);
			Misc.representativeOfBdd(cachedBdd, this);
		}
		return cachedBdd;
	}
	
	@Override
	public int hashCode() {
		return operand.hashCode()*3;
	}

	@Override
	public boolean equals(Object o) {
		if (!(o instanceof GOperator)) return false;
		else {
			return ((GOperator)o).operand.equals(this.operand);
		}
	}

	@Override
	public Formula negated() {
		return new FOperator(operand.negated());
	}

           	@Override
        public String toReversePolishString() {
                    return "G " + operand.toReversePolishString();
        }

	@Override
	public String toString() {
		if (strFormula == null) strFormula = "G " + operand.toString();
		return strFormula;
	}

	@Override
	public Tuple<Set<Formula>, Set<Formula>> recurrentFormulas(boolean accumulate) {
            Tuple<Set<Formula>, Set<Formula>> o = operand.recurrentFormulas(true);
            if ( operand.isProgressFormula() ) o.left.add(operand);
            return o;
	}

    @Override
    public Formula evaluateCurrentAssertions(Valuation valuation) {
            return this;
	}
    
	@Override
	public Formula removeConstantCurrentAssertions() {
		return this;
	}
	
	@Override
	public Formula removeX() {
		return this;
	}
	
	@Override
	public Formula unfold() {
		// U(F phi) = U(phi) \/ X F U(phi)
		return new Conjunction(operand.unfold(), new XOperator(new GOperator(operand)));
	}
	
	@Override
	public Formula unfoldNoG() {
		return this;
	}
	
	@Override
	public boolean isProgressFormula() {
		return false;
	}
	
	@Override
	public boolean containsG() {
		return true;
	}

        @Override
	public boolean hasSubformula(Formula f) {
            return this.equals(f) || operand.hasSubformula(f);
	}

        @Override
        public Formula removeXsFromCurrentBooleanAtoms() {
            return this;
        }

        @Override
	public Set<Formula> argumentsAlways() {
        	Set<Formula> r = operand.argumentsAlways();
        	r.add(operand);
            return r;
	}

    @Override
    public Set<Formula> argumentsFinsideG(boolean acc) {
        return operand.argumentsFinsideG(true);
    }

    public boolean untilOcurrs() {
	   return operand.untilOcurrs();
  	}
    
    public Formula toNNF() {
    	return new GOperator(operand.toNNF());
    }
}
