package rabinizer.exec;

import java.util.ArrayList;
import java.lang.*;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Stack;
import java.util.Map.Entry;





//import jdd.util.Array;
import rabinizer.formulas.Formula;

/**
 * 
 * @author Zuzana Komarkova
 *
 */

public class KDFAStates {
	
    public Set<Tuple<Tuple<Formula, KState>, List<List<Integer>>>> states = null;
    public Tuple<Tuple<Formula, KState>, List<List<Integer>>> initialState = null;
    public Map<Tuple<Tuple<Tuple<Formula, KState>, List<List<Integer>>>, Tuple<Tuple<Formula, KState>, List<List<Integer>>>>, ValuationSet> edges = null;

    public List<Tuple<Set<Tuple<Tuple<Formula, KState>, List<Integer>>>, List<Set<Tuple<Tuple<Formula, KState>, List<Integer>>>>>> condition = null;
    //public Set<Tuple<Map<Tuple<Tuple<Tuple<Formula, KState>,List<Integer>>, Tuple<Tuple<Formula, KState>,List<Integer>>>, ValuationSet>, Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet>>> conditionRabin = null;
    //public List< Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet>> conRabHelp= null;
    public Map<Formula, Formula> displayLabels = null;
    public List<Tuple<Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet>, List<Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet>>>> rCondition = null;
    //statesToNumbers - as generated, starting from 0
    public Map<Tuple<Tuple<Formula, KState>, List<List<Integer>>>,Integer> statesToNumbers=null;
    public Integer currentNumber;

    KDFAStates(KDFA kdfa, GeneralizedRabinAcceptanceCondition grCondition) {
    	Misc.verboseln("Rabin Automaton");
    	
    	displayLabels = kdfa.displayLabels;
    	
    	rCondition = new ArrayList<Tuple<Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet>, List<Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet>>>>();
    	for (Tuple<Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet>, List<Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet>>> pair : grCondition.condition){
    		List<Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet>> reachEventually = new ArrayList<Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet>>(pair.right.size());
    		reachEventually.addAll(pair.right);
    		rCondition.add(new Tuple<Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet>, List<Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet>>>(pair.left, reachEventually));
    	}
    	
    	
    	Misc.verboseln("Generalized Rabin Acceptance Condition on edges w/ fixed order");
    	Misc.verboseln(""+rCondition);
    	
    	// initial state
    	//Misc.verboseln("Number of copies of DFA: " + condition.size());
    	List<List<Integer>> iSCoords = new ArrayList<List<Integer>>(2*rCondition.size());
    	for (int i=0; i< 2*rCondition.size();i++){
			List<Integer> empty = new ArrayList<Integer>();
			empty.add(0,0);
			iSCoords.add(i,empty);
		}
		for (int i=0; i< rCondition.size();i++){
			List<Integer> empty2 = new ArrayList<Integer>();
			for(Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet> e : rCondition.get(i).right){
				empty2.add(0);
			}
			iSCoords.set(rCondition.size()+i,empty2);
		}
    	initialState = new Tuple<Tuple<Formula, KState>, List<List<Integer>>>(kdfa.initialState, iSCoords);

        states = new HashSet<Tuple<Tuple<Formula, KState>, List<List<Integer>>>>();
        states.add(initialState);
        statesToNumbers= new HashMap<Tuple<Tuple<Formula, KState>, List<List<Integer>>>,Integer>();
		currentNumber=0;
		statesToNumbers.put(initialState,currentNumber);
		currentNumber++;
        edges = new HashMap<Tuple<Tuple<Tuple<Formula, KState>, List<List<Integer>>>, Tuple<Tuple<Formula, KState>, List<List<Integer>>>>, ValuationSet>();
        Stack<Tuple<Tuple<Formula, KState>, List<List<Integer>>>> workstack = new Stack<Tuple<Tuple<Formula, KState>, List<List<Integer>>>>();
        workstack.push(initialState);

        int loopCount = 0;
        while (!workstack.empty()) {
        
        	Misc.verboseln("loop count: " + loopCount++);
        	Misc.verboseln("workstack : " + workstack);
        	Misc.verboseln("workcount : " + workstack.size());
        	
        	Tuple<Tuple<Formula, KState>, List<List<Integer>>> curr = workstack.pop();
            Misc.verboseln("Curr: " + curr);
            for (Map.Entry<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet> edge : kdfa.edges.entrySet()) {
            	if (edge.getKey().left.equals(curr.left)) {
	                Misc.verboseln("\tKDFA A edge: " + edge);
	                // successor state
	                
	                	
	                		
	                		for (Valuation v : edge.getValue()){
	                			List<List<Integer>> coords = new ArrayList<List<Integer>>(2*rCondition.size());
	                			for (int i=0; i< 2*rCondition.size();i++){
	                				List<Integer> empty = new ArrayList<Integer>();
	                				empty.add(0,0);
	                				coords.add(i,empty);
	                			}
	                			for (int i=0; i< rCondition.size();i++){
	                				List<Integer> empty2 = new ArrayList<Integer>();
	                				for(Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet> e : rCondition.get(i).right){
	                					empty2.add(0);
	                				}
	                				coords.set(rCondition.size()+i,empty2);
	                			}
	                			for (int i=0; i< rCondition.size();i++){
	                				if((rCondition.get(i).left.containsKey(edge.getKey())) && (rCondition.get(i).left.get(edge.getKey()).contains(v))){
	                					coords.get(i).set(0,1);
	                				}
	                				for(int j=0;j< rCondition.get(i).right.size();j++){
	                					Map<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet> e=rCondition.get(i).right.get(j);
	                					if((e.containsKey(edge.getKey())) && (e.get(edge.getKey()).contains(v))){
	                						coords.get(rCondition.size()+i).set(j,1);
	                					}
	                				
	                				}
	                				
	                	    	}
	                			Tuple<Tuple<Formula, KState>, List<List<Integer>>> state2 = new Tuple<Tuple<Formula, KState>, List<List<Integer>>>(edge.getKey().right, coords);
	                			Tuple<Tuple<Tuple<Formula, KState>, List<List<Integer>>>,Tuple<Tuple<Formula, KState>, List<List<Integer>>>> st = new Tuple<Tuple<Tuple<Formula, KState>, List<List<Integer>>>,Tuple<Tuple<Formula, KState>, List<List<Integer>>>>(curr,state2);
	                			if (!states.contains(state2)) {
		    	                	Misc.verboseln("\tNew state");
		    	                    states.add(state2);
		    	                    workstack.push(state2);
		    	                    statesToNumbers.put(state2,currentNumber);
		    						currentNumber++;
		    	                }
		                		
		                		if(edges.containsKey(st)){
		                			ValuationSet vs = edges.get(st);
		                			vs.add(v);
		                			edges.put(st, vs);
		                		}
		                		else{
		                			ValuationSet vs = new ValuationSet();
		                			vs.add(v);
		                			edges.put(st, vs);
		                		} 
	                		}
	                		
	                	}
	                

            	}
            }
        Misc.verboseln("Initial state: " + initialState);
        Misc.verboseln("Edges:         " + edges);
    }
    
    
    
    public String toDotty() {
		String dot = "digraph \"GenRabinAutomaton " + initialState +"\" {\n";
		//        Misc.verboseln("States       : " + states);
		//        Misc.verboseln("# states     : " + states.size());
		// The initial state in the set states may be different from the states in the set of edges. This can happen
		// because HashSet does not guarantee that two segment states that are equal have the same string representation.
		// To circumvent the problem, use the same string representation for rendering nodes and edges in the dotty file.
		Map<Tuple<Tuple<Formula, KState>, List<List<Integer>>>, Tuple<Tuple<Formula, KState>, List<List<Integer>>>> normalStates = new HashMap<Tuple<Tuple<Formula, KState>, List<List<Integer>>>, Tuple<Tuple<Formula, KState>, List<List<Integer>>>>();        
		for (Tuple<Tuple<Formula, KState>, List<List<Integer>>> s : states) {
			normalStates.put(s,s);
			Misc.verbose(",");
			Formula display = displayLabels.get(s.left.left);

			KState k = s.left.right;
			String display2 = "";
			for(SegmentAutomaton sa : k.state.keySet()){
				//List<Formula> fs = (List<Formula>) k.state.get(sa).keySet();
				display2 += "[";
				for(Formula f : k.state.get(sa).keySet())
					display2 += "" + sa.displayLabels.get(f) + ":" + k.state.get(sa).get(f) + " ";
				display2 += "] ";
			}
			String display3 = s.right.toString();
			if (s.equals(initialState)) {
				dot += "\nnode [shape=oval, label=\"" + display + " :: " + display2 + "::" + display3  + "\"]\"" + s + "\";\n";
			} else {
				dot += "\nnode [shape=rectangle, label=\"" + display + " :: " + display2 + "::" + display3  + "\"]\"" + s + "\";\n";
			}
		}
		for (Entry<Tuple<Tuple<Tuple<Formula, KState>, List<List<Integer>>>, Tuple<Tuple<Formula, KState>, List<List<Integer>>>>, ValuationSet> edge : edges.entrySet()) {
			Misc.verbose(".");
			dot += "\n\"" + normalStates.get(edge.getKey().left) + "\" -> \"" + normalStates.get(edge.getKey().right) + "\" [label=\"" + BDDService.BDDtoString(BDDService.valuationSetToBDD(edge.getValue())) + "\"];\n"; 
		}
		return dot + "}\n";
	}
    
    public String toPrism(int NrOfAp) {
		String a = "" + states.size() + " states "
				   + "(start " + statesToNumbers.get(initialState) + "), " 
				   + NrOfAp + " labels:\n";
		for (Entry<Tuple<Tuple<Tuple<Formula, KState>, List<List<Integer>>>, Tuple<Tuple<Formula, KState>, List<List<Integer>>>>, ValuationSet> edge : edges.entrySet()) {
			for (Valuation v : edge.getValue()) {
				a += " " + statesToNumbers.get(edge.getKey().left) + "-";
				a += v.toString();
				a += "->" + statesToNumbers.get(edge.getKey().right) + "\n";
			}
		}
		return a + "; ";
	}
}
