package rabinizer.formulas;

import java.util.*;

import net.sf.javabdd.*;
import rabinizer.exec.*;

/**
 * @author Andreas Gaiser & Ruslan Ledesma-Garza
 *
 */
public class Disjunction extends Formula {

	public Formula left, right;

	public Disjunction(Formula f1, Formula f2) {
		this.left = f1;
		this.right = f2;
	}

	public BDD bdd() {
		if (cachedBdd == null) {
			cachedBdd = left.bdd().or(right.bdd());
			Misc.representativeOfBdd(cachedBdd, this);
		}
		return cachedBdd;
	}

	@Override
	public int hashCode() {
		return 2*(left.hashCode()+right.hashCode());
	}

	@Override
	public boolean equals(Object o) {
		if (!(o instanceof Disjunction))
			return false;
		else 
			return ((Disjunction)o).left.equals(this.left) && ((Disjunction)o).right.equals(this.right);
	}

	@Override
	public Formula negated() {
		return new Conjunction(left.negated(), right.negated());
	}

	@Override
	public String toReversePolishString() {
		return "| " + left.toReversePolishString() + " " + right.toReversePolishString();
	}

	@Override
	public String toString() {
		if (strFormula == null) strFormula = "(" + left.toString() + " | " + right.toString() + ")";
		return strFormula;
	}

	@Override
	public Tuple<Set<Formula>, Set<Formula>> recurrentFormulas(boolean accumulate) {
		Tuple<Set<Formula>, Set<Formula>> l = left.recurrentFormulas(accumulate);
		Tuple<Set<Formula>, Set<Formula>> r = right.recurrentFormulas(accumulate);
		l.left.addAll(r.left);
		l.right.addAll(r.right);
		return l;
	}

	@Override
	public Formula evaluateCurrentAssertions(Valuation valuation) {
		return new Disjunction(left.evaluateCurrentAssertions(valuation),
				right.evaluateCurrentAssertions(valuation));	
	}

	@Override
	public Formula removeConstantCurrentAssertions() {
		Formula new_left = left.removeConstantCurrentAssertions();
		if (new_left instanceof BooleanConstant) {
			if (((BooleanConstant)new_left).value) {
				return new BooleanConstant(true);
			} else {
				return right.removeConstantCurrentAssertions(); 
			}
		} else { 
			Formula new_right = right.removeConstantCurrentAssertions();
			if (new_right instanceof BooleanConstant) {
				if (((BooleanConstant)new_right).value){
					return new BooleanConstant(true);
				} else {
					return new_left;
				}
			} else {
				return new Disjunction(new_left, new_right);
			}
		}
	}

	@Override
	public Formula removeX() {
		return new Disjunction(left.removeX(), right.removeX());
	}

	@Override
	public Formula unfold() {
		return new Disjunction(left.unfold(), right.unfold());
	}

	@Override
	public Formula unfoldNoG() {
		return new Disjunction(left.unfoldNoG(), right.unfoldNoG());
	}

	@Override
	public boolean isProgressFormula() {
		return left.isProgressFormula() || right.isProgressFormula();
	}

	@Override
	public boolean hasSuspendableG(Formula f) {
		return left.hasSuspendableG(f) && right.hasSuspendableG(f);
	}
	
	@Override
	public boolean isVeryDifferentFrom(Formula f) {
		return left.isVeryDifferentFrom(f) || right.isVeryDifferentFrom(f);
	}
	
	@Override
	public boolean isUnfoldOfF() {
		if (right instanceof XOperator)
			if (((XOperator) right).operand instanceof FOperator)
				return true;
		return false;
	}
	
	@Override
	public boolean containsG() {
		return left.containsG() || right.containsG();
	}

	@Override
	public boolean hasSubformula(Formula f) {
		return this.equals(f) || left.hasSubformula(f) || right.hasSubformula(f);
	}

	@Override
	public Formula removeXsFromCurrentBooleanAtoms() {
		return new Disjunction(left.removeXsFromCurrentBooleanAtoms(), right.removeXsFromCurrentBooleanAtoms());
	}

	@Override
	public Set<Formula> argumentsAlways() {
		Set<Formula> r = left.argumentsAlways();
		r.addAll(right.argumentsAlways());
		return r;
	}

	@Override
	public Set<Formula> argumentsFinsideG(boolean acc) {
		Set<Formula> r = left.argumentsFinsideG(acc);
		r.addAll(right.argumentsFinsideG(acc));
		return r;
	}

	public boolean untilOcurrs() {
		return left.untilOcurrs() || right.untilOcurrs();
	}

	public Formula toNNF() {
		return new Disjunction(left.toNNF(), right.toNNF());
	}
}
