package rabinizer.formulas;

import java.util.*;

import net.sf.javabdd.BDD;
import rabinizer.exec.*;

public class Negation extends Formula {

	public Formula operand;

	public Negation(Formula f) {
		this.operand = f;
	}
	
	public BDD bdd() {            // negation of ATOMIC PROPOSITIONS only
		if (cachedBdd == null) {
			Formula booleanAtom = new Negation(operand.representative());
			int bddVar = Misc.bijectionBooleanAtomBddVar.id(booleanAtom);
			if (Misc.bddFactory.varNum() <= bddVar)	Misc.bddFactory.extVarNum(1);
			cachedBdd = Misc.bddFactory.ithVar(bddVar);
			Misc.representativeOfBdd(cachedBdd, this);
		}
		return cachedBdd;
	}
	
	@Override
	public int hashCode() {
		return operand.hashCode() * 2;
	}

	@Override
	public boolean equals(Object o) {
		if (!(o instanceof Negation))
			return false;
		else
			return ((Negation)o).operand.equals(this.operand);
	}
	
	@Override
	public Formula negated() {
		return new Negation(operand.negated());
	}

   	@Override
    public String toReversePolishString() {
   		return "! " + operand.toReversePolishString();
   	}

	@Override
	public String toString() {
		if (strFormula == null) strFormula = "! " + operand.toString();
		return strFormula;
	}

	@Override
	public Tuple<Set<Formula>, Set<Formula>> recurrentFormulas(boolean accumulate) {
		return operand.recurrentFormulas(accumulate);
	}

    @Override
    public Formula evaluateCurrentAssertions(Valuation valuation) {
        Formula o = operand.evaluateCurrentAssertions(valuation);
        if (! (o instanceof BooleanConstant)) 
            Rabinizer.errorMessageAndExit("Negation.evaluateCurrentAssertions: the given formula is not in Negation Normal Form.");
        return o.negated();
    }
	
	@Override
	public Formula removeConstantCurrentAssertions() {
		return this;
	}
	
	@Override
	public Formula removeX() {
		return operand;
	}

	@Override
	public Formula unfold() {
		return this;
	}

	@Override
	public Formula unfoldNoG() {
		return this;
	}
	
	@Override
	public boolean isProgressFormula() {
		return operand.isProgressFormula();
	}

        @Override
	public boolean hasSubformula(Formula f) {
            return this.equals(f) || operand.hasSubformula(f);
	}

        @Override
        public Formula removeXsFromCurrentBooleanAtoms() {
            if (operand.isProgressFormula()) return this;
            else return operand.removeXsFromCurrentBooleanAtoms();
        }

    @Override
	public Set<Formula> argumentsAlways() {
            return operand.argumentsAlways();
	}

        @Override
    public Set<Formula> argumentsFinsideG(boolean acc) {
        return operand.argumentsFinsideG(acc);
    }
        
    public boolean untilOcurrs() {
    	return operand.untilOcurrs();
    }
    
    public Formula toNNF() {
    	if (operand instanceof Literal) {
    		Literal l = new Literal((Literal)operand);
    		l.negated = !l.negated;
    		return l;
    	} else if (operand instanceof BooleanConstant) {
    		return new BooleanConstant(!((BooleanConstant)operand).value);
    	} else if (operand instanceof FOperator) {
            FOperator f = (FOperator) operand;
    		return new GOperator((new Negation(f.operand)).toNNF());
    	} else if (operand instanceof GOperator) {
            GOperator g = (GOperator) operand;
            return new FOperator((new Negation(g.operand)).toNNF());
    	} else if (operand instanceof XOperator) {
            Negation n = (Negation) operand;
            return new XOperator((new Negation(n.operand)).toNNF());
    	} else if (operand instanceof Negation) {
    		return operand;
    	} else if (operand instanceof UOperator) {
    		UOperator u = (UOperator)operand;
                Formula l = (new UOperator(new Negation(u.right), new Conjunction(new Negation(u.left), new Negation(u.right)))).toNNF();
                Formula r = (new GOperator(new Negation(u.right))).toNNF();
    		return new Disjunction(l, r);
    	} else if (operand instanceof Conjunction) {
    		Conjunction c = (Conjunction)operand;
    		Formula l = (new Negation(c.left)).toNNF();
    		Formula r = (new Negation(c.right)).toNNF();
    		return new Disjunction(l, r);
    	} else if (operand instanceof Disjunction) {
    		Disjunction d = (Disjunction)operand;
    		Formula l = (new Negation(d.left)).toNNF();
    		Formula r = (new Negation(d.right)).toNNF();
    		return new Conjunction(l, r);
    	} else {
    		throw new Error("Negation.toNNF: Unknown formula: "+operand);
    	} 	
    }
}
