package rabinizer.formulas;

import java.util.*;

import net.sf.javabdd.*;
import rabinizer.exec.*;

/**
 * Represents a until formula.
 * @author Andreas & Ruslan
 *
 */
public class UOperator extends Formula {

	public Formula left, right;

	public UOperator(Formula f1, Formula f2) {
		this.left = f1;
		this.right = f2;
	}
	
	public BDD bdd() {
		if (cachedBdd == null) {
			Formula booleanAtom = new UOperator(
					left.representative(),
					right.representative()
					);
			int bddVar = Misc.bijectionBooleanAtomBddVar.id(booleanAtom);
			if (Misc.bddFactory.varNum() <= bddVar) Misc.bddFactory.extVarNum(bddVar);
			cachedBdd = Misc.bddFactory.ithVar(bddVar);
			Misc.representativeOfBdd(cachedBdd, this);
		}
		return cachedBdd;
	}

	@Override
	public int hashCode() {
		return 3 * (left.hashCode() + right.hashCode());
	}

	@Override
	public boolean equals(Object o) {
		if (!(o instanceof UOperator))
			return false;
		else
			return ((UOperator)o).left.equals(this.left) && ((UOperator)o).right.equals(this.right);
	}
	
	@Override
	public Formula negated() {
		throw new Error("UOperator.negated(): NYI");
	}

           	@Override
        public String toReversePolishString() {
                    return "U " + left.toReversePolishString() + " " + right.toReversePolishString();
        }
    
    @Override
	public String toString() {
    	if (strFormula == null) strFormula = "(" + left.toString() + " U " + right.toString() + ")";
    	return strFormula;
	}

	@Override
	public Tuple<Set<Formula>, Set<Formula>> recurrentFormulas(boolean accumulate) {
            Tuple<Set<Formula>, Set<Formula>> l = left.recurrentFormulas(accumulate);
            Tuple<Set<Formula>, Set<Formula>> r = right.recurrentFormulas(accumulate);
            l.left.addAll(r.left);
            l.right.addAll(r.right);
            return l;
	}

    @Override
    public Formula evaluateCurrentAssertions(Valuation valuation) {
            return this;
	}
    
	@Override
	public Formula removeConstantCurrentAssertions() {
		return this;
	}	
	
	@Override
	public Formula removeX() {
		return this;
	}

	@Override
	public Formula unfold() {
		// unfold(a U b) = unfold(b) v (unfold(a) ^ X (a U b))
		return new Disjunction (right.unfold(), new Conjunction(left.unfold(), new XOperator(this)));
	}

	@Override
	public Formula unfoldNoG() {
		// unfold(a U b) = unfold(b) v (unfold(a) ^ X (a U b))
		return new Disjunction (right.unfoldNoG(), new Conjunction(left.unfoldNoG(), new XOperator(this)));
	}
	
	@Override
	public boolean isProgressFormula() {
		return right.isProgressFormula();
	}
	
	@Override
	public boolean containsG() {
		return left.containsG() || right.containsG();
	}

        @Override
	public boolean hasSubformula(Formula f) {
            return this.equals(f) || left.hasSubformula(f) || right.hasSubformula(f);
	}

        @Override
        public Formula removeXsFromCurrentBooleanAtoms() {
            return this;
        }

           @Override
	public Set<Formula> argumentsAlways() {
           	Set<Formula> r = left.argumentsAlways();
           	r.addAll(right.argumentsAlways());
           return r;
	}

                   @Override
            public Set<Formula> argumentsFinsideG(boolean acc) {
               Set<Formula> r = left.argumentsFinsideG(acc);
               r.addAll(right.argumentsFinsideG(acc));
               return r;
	}


    public boolean untilOcurrs() {
    	return true;
    }
    
    public Formula toNNF() {
    	return new UOperator(left.toNNF(), right.toNNF());
    }
}
