package tests;

import static org.junit.Assert.*;

import java.io.StringReader;
import java.util.*;

import org.junit.Test;

import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

import rabinizer.exec.*;
import rabinizer.formulas.*;
import rabinizer.parser.LTLParser;

@RunWith(value = Parameterized.class)
public class SegmentAutomatonStep {

	private List<String> rawCurrentSegmentState;
	private Valuation valuation;
	private List<String> rawExpectedSegmentState;
	
	public SegmentAutomatonStep(List<String> input, List<Boolean> values, List<String> expected) {
		this.rawCurrentSegmentState = input;
		this.valuation = new Valuation(values);
		this.rawExpectedSegmentState = expected;
	}

    @Parameters
	public static Collection<Object[]> data() {
        Object[][] data = new Object[][] {
        		{ Arrays.asList("(a & X b) | (b & X X a)"), Arrays.asList(true, false), Arrays.asList("b", "(a & X b) | (b & X X a)") },
        		{ Arrays.asList("(a & X b) | (b & X X a)"), Arrays.asList(false, true), Arrays.asList("X a", "(a & X b) | (b & X X a)") },
        		{ Arrays.asList("(a & X b) | (b & X X a)"), Arrays.asList(true, true), Arrays.asList("b | X a", "(a & X b) | (b & X X a)") },
        		{ Arrays.asList("b | X a"), Arrays.asList(true, true), Arrays.asList("true") },
        		{ Arrays.asList("X a", "b | X a"), Arrays.asList(true, true), Arrays.asList("a", "true") },
        		{ Arrays.asList("b", "X a", "b | X a"), Arrays.asList(true, true), Arrays.asList("a", "true") },
        		{ Arrays.asList("b"), Arrays.asList(true, false), Arrays.asList("false") }, // experiment: include the false state
        };
        return Arrays.asList(data);
    }

	@Test
	public void pushTest() {
		System.out.println(this.getClass().getCanonicalName()+" : "+rawCurrentSegmentState);
		
		LTLParser parser = new LTLParser(new StringReader("(a & X b) | (b & X X a)"));
		Formula initialState = null;
		try { initialState = parser.parse(); }
		catch (Exception e) { assertTrue(false); }
		
		Misc.initializeValuations(Misc.bijectionIdAtom.size());
		SegmentAutomaton sA = new SegmentAutomaton(initialState, Misc.valuations);
		
		Set<Formula> currentSegmentState = new HashSet<Formula>();
		for (String state : rawCurrentSegmentState) {
			parser.ReInit(new StringReader(state));
			try {  currentSegmentState.add(parser.parsePreviousAtoms()); }
			catch (Exception e) { assertTrue(false); }
		}
		
		Set<Formula> expectedSegmentState = new HashSet<Formula>();
		for (String state : rawExpectedSegmentState) {
			parser.ReInit(new StringReader(state));
			try {  expectedSegmentState.add(parser.parsePreviousAtoms()); }
			catch (Exception e) { assertTrue(false); }
		}
		
		Set<Formula> actualSegmentState = sA.step(currentSegmentState, valuation);
		
		System.out.println("expected: " + expectedSegmentState);
		System.out.println("actual  : " + actualSegmentState);
		
		assertTrue(actualSegmentState.equals(expectedSegmentState));
	}
}
